package trask.app.life;

import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.beans.*;

public class LifeApplet extends Applet
{
  LifePicture lifePicture;
  CalculationDisplay calculationDisplay;
  LifeColorControl lifeColorControl;
  LifeDimensionControls lifeDimensionControls;
  Button clearButton;
  Button recalculateButton;
  Button detailsButton;

  Checkbox disjointCheckbox;
  Checkbox overlappingCheckbox;

  public void init()
  {
		ClassLoader classLoader = getClass().getClassLoader();
	
    // instantiate beans

    // the way it is supposed to work

    try {
      lifePicture = (LifePicture) Beans.instantiate(classLoader,
												  "trask.app.life.LifePicture");
      calculationDisplay = (CalculationDisplay) Beans.instantiate(classLoader,
							                       "trask.app.life.CalculationDisplay");
      lifeColorControl = (LifeColorControl) Beans.instantiate(classLoader,
                                                   "trask.app.life.LifeColorControl");
      lifeDimensionControls = (LifeDimensionControls) Beans.instantiate(classLoader,
                                                   "trask.app.life.LifeDimensionControls");
      clearButton = (Button) Beans.instantiate(classLoader, "java.awt.Button");
      recalculateButton = (Button) Beans.instantiate(classLoader, "java.awt.Button");
      detailsButton = (Button) Beans.instantiate(classLoader, "java.awt.Button");
    }
    catch (Exception exc) {
      System.out.println(exc);
    }

    // temporary (?)

    CheckboxGroup group = new CheckboxGroup();
    disjointCheckbox = new Checkbox("Disjoint", false, group);
    overlappingCheckbox = new Checkbox("Overlapping", true, group);
    calculationDisplay.setUseDisjointSubBlocks(disjointCheckbox.getState());

    disjointCheckbox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent event) {
        calculationDisplay.setUseDisjointSubBlocks(disjointCheckbox.getState());
      }});
    overlappingCheckbox.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent event) {
        calculationDisplay.setUseDisjointSubBlocks(!overlappingCheckbox.getState());
      }});


    // wire beans

    lifeColorControl.addColorChangeListener(new ColorChangeListener() {
      public void colorChanged(ColorChangeEvent event) {
        lifePicture.setCurrentColor(event.getColor());
      }});

    lifePicture.addPropertyChangeListener(new PropertyChangeListener() {
      public void propertyChange(PropertyChangeEvent event) {
        if (event.getPropertyName().equals("pictureDim")) {
          lifeDimensionControls.setPictureDim((Dimension) event.getNewValue());
        } else if (event.getPropertyName().equals("elementDim")) {
          lifeDimensionControls.setElementDim((Dimension) event.getNewValue());
        }
      }});

    lifeDimensionControls.addDimensionChangeListener(new DimensionChangeListener() {
      public void dimensionChanged(DimensionChangeEvent event) {
        lifePicture.setElementDim(event.getElementDim());
        lifePicture.setPictureDim(event.getPictureDim());
      }});

    clearButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        lifePicture.clearPicture();
      }});

		recalculateButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        calculationDisplay.calculateLife(lifePicture.getPicture(), lifePicture.getPictureDim());
        invalidate();  //?
        validate();       //?
      }});

    detailsButton.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent event) {
        calculationDisplay.setCalculationDetailsVisible(true);
      }});

    // initialize beans

    Color[] colors = new Color[] {Color.white, Color.lightGray,
                                       Color.darkGray.brighter(), Color.black};
    String[] labels = new String[] {"White", "Light Gray", "Dark Gray", "Black"};
    lifeColorControl.setColors(colors);
    lifeColorControl.setLabels(labels);
    lifeColorControl.setCurrentColorIndex(0);

//    lifePicture.setCurrentColor(lifeColorControls.getCurrentColor());
    lifePicture.newPicture(new Dimension(6, 6), new Dimension(20, 20));

    clearButton.setLabel("Clear");
    recalculateButton.setLabel("Recalculate");
    detailsButton.setLabel("Details");

    calculationDisplay.calculateLife(lifePicture.getPicture(), lifePicture.getPictureDim());

    // layout beans

    setBackground(Color.white);
    setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();

    // life canvas
    constraints.gridx = 0;
    constraints.gridy = 0;
    constraints.insets = new Insets(0, 10, 10, 10);
    add(lifePicture, constraints);

    // calculation display
    constraints.gridx = 0;
    constraints.gridy = 1;
    add(calculationDisplay, constraints);

    // life controls
    Panel controlPanel = new Panel();
    controlPanel.setLayout(new GridBagLayout());
    constraints = new GridBagConstraints();
    constraints.anchor = GridBagConstraints.WEST;

    constraints.gridx = 0;
    constraints.gridy = 0;
    controlPanel.add(lifeColorControl, constraints);

    constraints.gridx = 0;
    constraints.gridy = 1;
    constraints.insets = new Insets(5, 0, 5, 0);
    constraints.ipadx = 35;
    controlPanel.add(clearButton, constraints);
    constraints.ipadx = 0;

/*
    constraints.gridx = 0;
    constraints.gridy = 2;
    constraints.ipadx = 27;
    controlPanel.add(resizeButton, constraints);
*/
    constraints.gridx = 0;
    constraints.gridy = 2;
    controlPanel.add(recalculateButton, constraints);

    constraints.gridx = 0;
    constraints.gridy = 3;
    constraints.ipadx = 26;
    controlPanel.add(detailsButton, constraints);
    constraints.ipadx = 0;
    constraints.insets = new Insets(0, 0, 0, 0);

    constraints.gridx = 0;
    constraints.gridy = 4;
    controlPanel.add(disjointCheckbox, constraints);

    constraints.gridx = 0;
    constraints.gridy = 5;
    controlPanel.add(overlappingCheckbox, constraints);

    constraints.gridx = 1;
    constraints.gridy = 0;
    constraints.gridheight = 2;
    constraints.anchor = GridBagConstraints.NORTH;
    add(controlPanel, constraints);
  }

  public void paint(Graphics g)
  {
    super.paint(g);
    g.drawRect(0, 0, getSize().width - 1, getSize().height - 1);
  }

}


