package trask.app.life;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.io.*;
import java.net.URL;
import java.awt.image.*;

public class LifePicture extends Component
implements MouseListener, MouseMotionListener
{
  private Dimension pictureDim = new Dimension(0, 0);
  private Dimension elementDim = new Dimension(0, 0);

  private int inset = 3;
  private Color currentColor = Color.white;

  private Color[][] picture;

  private Vector pictureChangeListeners = new Vector();

  private PropertyChangeSupport changes = new PropertyChangeSupport(this);

  // constructor

  public LifePicture()
  {
    addMouseListener(this);
    addMouseMotionListener(this);
  }

	// get and set methods
	
  public void setPictureDim(Dimension picDim)
  {
    if (pictureDim.width == picDim.width && pictureDim.height == picDim.height) {
      return;
    }

    Dimension newPictureDim = picDim;
    Color[][] newPicture = new Color[newPictureDim.width][newPictureDim.height];

    for (int i = 0; i < newPictureDim.width; i++) {
      for (int j = 0; j < newPictureDim.height; j++) {
        if (i < pictureDim.width && j < pictureDim.height) {
          newPicture[i][j] = picture[i][j];
        } else {
          newPicture[i][j] = currentColor;
        }
      }
    }

    picture = newPicture;

		Dimension oldPictureDim = pictureDim;
    pictureDim = new Dimension(newPictureDim);
    changes.firePropertyChange("pictureDim", oldPictureDim, pictureDim);

    invalidate();
    getRootComponent().validate();
//    getFrame().repaint();
  }

  // get and set methods

  public Color[][] getPicture()
  {
    return picture;
  }

  public Dimension getPictureDim()
  {
    return new Dimension(pictureDim);
  }

	public Dimension getElementDim()
  {
    return new Dimension(elementDim);
  }

	public void setElementDim(Dimension eltDim)
  {
    if (elementDim.width == eltDim.width && elementDim.height == eltDim.height) {
      return;
    }

    Dimension oldElementDim = elementDim;
    elementDim = new Dimension(eltDim);
    changes.firePropertyChange("elementDim", oldElementDim, elementDim);

    invalidate();
    getRootComponent().validate();
  }

  public Color getCurrentColor()
  {
    return currentColor;
  }

  public void setCurrentColor(Color color)
  {
    Color oldCurrentColor = currentColor;
    currentColor = color;
    changes.firePropertyChange("currentColor", oldCurrentColor, currentColor);
  }

  // important methods

  public void changeElement(int i, int j)
  {
    changeElement(i, j, currentColor);
  }

  public void changeElement(int i, int j, Color color)
  {
    if (picture[i][j] != color) {
      picture[i][j] = color;

      Graphics g = getGraphics();
      g.setColor(color);
      g.fillRect(inset + i * elementDim.width, inset + j * elementDim.height,
                 elementDim.width, elementDim.height);
      g.dispose();
    }
  }

  public void newPicture(Dimension picDim, Dimension eltDim)
  {
    picture = new Color[picDim.width][picDim.height];

    setPictureDim(picDim);
    setElementDim(eltDim);
    clearPicture();
  }

  public void clearPicture()
  {
    for (int i = 0; i < pictureDim.width; i++) {
      for (int j = 0; j < pictureDim.height; j++) {
        picture[i][j] = currentColor;
      }
    }
    repaint();
  }

  public void loadImage(URL imageURL, Color[] colorScheme)
  {
    Image image = Toolkit.getDefaultToolkit().getImage(imageURL);
    MediaTracker tracker = new MediaTracker(this);
    tracker.addImage(image, 0);
    try {
      tracker.waitForID(0);
    }
    catch (InterruptedException exc) {
    }

    int width = image.getWidth(this);
    int height = image.getHeight(this);

    picture = new Color[width][height];
    setElementDim(new Dimension(1, 1));
    setPictureDim(new Dimension(width, height));

    int[] pixels = new int[width * height];
    PixelGrabber pg = new PixelGrabber(image, 0, 0, width, height, pixels, 0, width);
    try {
      pg.grabPixels();
    }
    catch (InterruptedException exc) {
    }

    for (int j = 0; j < height; j++) {
      for (int i = 0; i < width; i++) {
        picture[i][j] = getColorFromScheme(pixels[width * j + i], colorScheme);
      }
    }
    invalidate();
    getRootComponent().validate();
  }

  private Color getColorFromScheme(int pixel, Color[] colorScheme)
  {
    /* temporary, only designed for gray scale colorSchemes */

    int red = (pixel >> 16) & 0xff;
    int green = (pixel >> 8) & 0xff;
    int blue = pixel & 0xff;

    Color bestColor = null;
    double bestDColor = 0;
    for (int i = 0; i < colorScheme.length; i++) {
      int r = colorScheme[i].getRed();
      int g = colorScheme[i].getGreen();
      int b = colorScheme[i].getBlue();
      int dColor = Math.abs(red - r) + Math.abs(green - g) + Math.abs(blue - b);

      if (dColor < bestDColor || bestColor == null) {
        bestColor = colorScheme[i];
        bestDColor = dColor;
      }
    }

    return bestColor;
  }

  public void savePicture(File pictureFile)
  {
    System.out.println("savePicture");
    try {
      ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(pictureFile));
      out.writeObject(picture);
      out.writeObject(pictureDim);
      out.writeObject(elementDim);
      out.close();
    }
    catch (IOException exc) {
      System.out.println(exc);
    }
  }

  public void loadPicture(File pictureFile, Color[] colorScheme)
  {
    try {
      ObjectInputStream in = new ObjectInputStream(new FileInputStream(pictureFile));
      picture = (Color[][]) in.readObject();
      setPictureDim((Dimension) in.readObject());
      setElementDim((Dimension) in.readObject());
      in.close();

      for (int i = 0; i < pictureDim.width; i++ ) {
        for (int j = 0; j < pictureDim.height; j++) {
          picture[i][j] = getColorFromScheme(picture[i][j].getRGB(), colorScheme);
        }
      }
      invalidate();
      getRootComponent().validate();
    }
    catch (Exception exc) {
      System.out.println(exc);
    }
  }

  public Component getRootComponent()
  {
    Component c = this;

    while (c.getParent() != null) {
      c = c.getParent();
    }
    return (Component) c;
  }


  // PropertyChange methods

  public void addPropertyChangeListener(PropertyChangeListener l)
  {
    changes.addPropertyChangeListener(l);
  }

  public void removePropertyChangeListener(PropertyChangeListener l)
  {
    changes.removePropertyChangeListener(l);
  }

  // update and paint methods

  public void update(Graphics g)
  {
    paint(g);
  }

  public void paint(Graphics g)
  {
    for (int i = 0; i < pictureDim.width; i++) {
      for (int j = 0; j < pictureDim.height; j++) {
        g.setColor(picture[i][j]);
        g.fillRect(inset + i * elementDim.width, inset + j * elementDim.height,
                   elementDim.width, elementDim.height);
      }
    }
    g.setColor(Color.black);
    g.drawRect(0, 0, getSize().width - 1, getSize().height - 1);
  }

  // getPreferredSize and getMinimumSize methods

  public Dimension getPreferredSize()
  {
    int width = pictureDim.width * elementDim.width;
    int height = pictureDim.height * elementDim.height;

    return new Dimension(width + 2 * inset, height + 2 * inset);
  }

  public Dimension getMinimumSize()
  {
    return getPreferredSize();
  }

  // MouseListener and MouseMotionListener methods

  public void mousePressed(MouseEvent event)
  {
    int i = event.getX() / elementDim.width;
    int j = event.getY() / elementDim.height;

    if (i >= 0 && i < pictureDim.width && j >= 0 && j < pictureDim.height) {
      changeElement(i, j);
    }
  }

  public void mouseDragged(MouseEvent event)
  {
    int i = event.getX() / elementDim.width;
    int j = event.getY() / elementDim.height;

    if (i >= 0 && i < pictureDim.width && j >= 0 && j < pictureDim.height) {
      changeElement(i, j);
    }
  }

  public void mouseReleased(MouseEvent event) {}
  public void mouseClicked(MouseEvent event) {}
  public void mouseEntered(MouseEvent event) {}
  public void mouseExited(MouseEvent event) {}
  public void mouseMoved(MouseEvent event) {}

}

